@@ -14,39 +14,46 @@ import time
 import sys
 import os
 
-if sys.version_info < (3, 0):
-    import cPickle as pickle  # Python2
-else:
-    import _pickle as pickle  # Python3
+import _pickle as pickle  # Python3
 
-try:
-    # This works in Python2.7...
-    from Queue import Empty, Full
-except ImportError:
-    # This works in Python3.x...
-    from queue import Empty, Full
+# This works in Python3.x...
+from queue import Empty, Full
 
 from colorama import init as color_init
 from colorama import Fore, Style
 
 """ Polymer.py - Manage parallel tasks
-     Copyright (C) 2015-2020 David Michael Pennington
+     Copyright (C) 2021-2022 David Michael Pennington
+     Copyright (C) 2020-2021 David Michael Pennington at Cisco Systems
+     Copyright (C) 2019      David Michael Pennington at ThousandEyes
+     Copyright (C) 2015-2019 David Michael Pennington at Samsung Data Services
 
-     This program is free software: you can redistribute it and/or modify
-     it under the terms of the GNU General Public License as published by
-     the Free Software Foundation, either version 3 of the License, or
-     (at your option) any later version.
+Redistribution and use in source and binary forms, with or without
+modification, are permitted provided that the following conditions are
+met:
 
-     This program is distributed in the hope that it will be useful,
-     but WITHOUT ANY WARRANTY; without even the implied warranty of
-     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-     GNU General Public License for more details.
+1.  Redistributions of source code must retain the above copyright
+    notice, this list of conditions and the following disclaimer.
 
-     You should have received a copy of the GNU General Public License
-     along with this program.  If not, see <http://www.gnu.org/licenses/>.
+2.  Redistributions in binary form must reproduce the above copyright
+    notice, this list of conditions and the following disclaimer in the
+    documentation and/or other materials provided with the distribution.
 
-     If you need to contact the author, you can do so by emailing:
-     mike [~at~] pennington [/dot\] net
+3.  Neither the name of the copyright holder nor the names of its
+    contributors may be used to endorse or promote products derived from
+    this software without specific prior written permission.
+
+THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
+PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
+TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
+LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 """
 
 class SharedCounter(object):
@@ -57,6 +64,7 @@ class SharedCounter(object):
     second process may read the old value before the new one is written by the
     first process. The solution is to use a multiprocessing.Lock to guarantee
     the atomicity of the modifications to Value.
+
     This class comes almost entirely from Eli Bendersky's blog:
     http://eli.thegreenplace.net/2012/01/04/shared-counter-with-pythons-multiprocessing/
     """
@@ -97,31 +105,55 @@ class py23_mp_queue(mpq.Queue):
     #    https://github.com/vterron/lemon/blob/master/util/queue.py
 
     def __init__(self, *args, **kwargs):
-        if sys.version_info >= (3, 4, 0):
-            super(py23_mp_queue, self).__init__(
-                *args, ctx=multiprocessing.get_context("spawn"), **kwargs
-            )
-        else:
-            super(py23_mp_queue, self).__init__(*args, **kwargs)
-        self.size = SharedCounter(0)
+        # Use ctx argument if using Python3.4+
+        try:
+            if sys.version_info >= (3, 4, 0):
+                super(py23_mp_queue, self).__init__(
+                    *args, ctx=multiprocessing.get_context("spawn"), **kwargs
+                )
+            else:
+                super(py23_mp_queue, self).__init__(*args, **kwargs)
+
+        except Exception as ee:
+            raise(ee)
+
+        if not sys.platform=='darwin':
+            self.size = SharedCounter(0)
 
     def put(self, *args, **kwargs):
         # Infinite recursion possible if we don't use super() here
-        super(py23_mp_queue, self).put(*args, **kwargs)
-        self.size.increment(1)
+        try:
+            super(py23_mp_queue, self).put(*args, **kwargs)
+
+        except Full:
+            pass
+
+        if not sys.platform=='darwin':
+            self.size.increment(1)
 
     def get(self, *args, **kwargs):
-        item = super(py23_mp_queue, self).get(*args, **kwargs)
-        self.size.increment(-1)
-        return item
+        if not sys.platform=='darwin':
+            self.size.increment(-1)
+        try:
+            item = super(py23_mp_queue, self).get(*args, **kwargs)
+            return item
+
+        except Empty:
+            return {}
 
     def qsize(self):
         """ Reliable implementation of multiprocessing.Queue.qsize() """
-        return self.size.value
+        if not sys.platform=='darwin':
+            return self.size.value
+        else:
+            raise NotImplementedError
 
     def empty(self):
         """ Reliable implementation of multiprocessing.Queue.empty() """
-        return not self.qsize()
+        if not sys.platform=='darwin':
+            return not self.qsize()
+        else:
+            raise NotImplementedError
 
     def clear(self):
         """ Remove all elements from the Queue. """
@@ -142,7 +174,7 @@ class Worker(object):
 
         try:
             self.message_loop(todo_q, done_q)  # do work
-        except:
+        except Exception:
             if self.task is not None:
                 self.task.task_stop = time.time()  # Seconds since epoch
 
@@ -320,7 +352,7 @@ class Worker(object):
             except Full:
                 time.sleep(0.1)
             ## Disable extraneous error handling...
-            except:
+            except Exception:
                 if self.task is not None:
                     self.task.task_stop = time.time()  # Seconds since epoch
                 # Handle all other errors here...
@@ -582,7 +614,7 @@ class TaskMgr(object):
                             try:
                                 # Delete the worker assignment...
                                 self.worker_assignments.pop(w_id)
-                            except:
+                            except Exception:
                                 pass
                             self.retval.add(task)  # Add result to retval
 
@@ -600,7 +632,7 @@ class TaskMgr(object):
                 if self.log_level >= 0:
                     self.log.info(stats.log_message)
 
-            # Adaptive loop delay
+            # Adaptive loop delay unless on Mac OSX... OSX delay is constant...
             delay = self.calc_wait_time(stats.exec_times)
             time.sleep(delay)
 
@@ -619,12 +651,18 @@ class TaskMgr(object):
 
     def calc_wait_time(self, exec_times):
         num_samples = float(len(exec_times))
-        if num_samples > 0.0:
+
+        # Work around Mac OSX problematic queue.qsize() implementation...
+        if sys.platform=='darwin':
+            wait_time = 0.00001  # 10us delay to avoid worker / done_q race
+
+        elif num_samples > 0.0:
             # NOTE:  OSX doesn't implement queue.qsize(), I worked around the
             # problem
             queue_size = max(self.done_q.qsize(), 1.0) + max(self.todo_q.qsize(), 1.0)
             min_task_time = min(exec_times)
             wait_time = min_task_time / queue_size
+
         else:
             wait_time = 0.00001  # 10us delay to avoid worker / done_q race
 
